package sanketvekariya.flutter_flexible_toast;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.PorterDuff;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import java.util.Objects;

import androidx.cardview.widget.CardView;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * FlutterFlexibleToastPlugin is totally customizable toast accessible from Flutter Side Code.
 */
public class FlutterFlexibleToastPlugin implements MethodCallHandler {

    private Context mContext;
    private Toast mToast;

    private FlutterFlexibleToastPlugin(Context context) {
        mContext = context;
    }

    /**
     * Registration of Method Channel with it's name "flutter_flexible_toast"
     *
     * @param registrar
     */
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_flexible_toast");
        channel.setMethodCallHandler(new FlutterFlexibleToastPlugin(registrar.context()));
    }

    /**
     * Override method onMethodCall for Redesigning entire toast logic
     *
     * @param call
     * @param result
     */
    @Override
    public void onMethodCall(MethodCall call, final Result result) {

        /*
          View Initialized for android ui code
         */
        LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        @SuppressLint("InflateParams") View layout = inflater.inflate(R.layout.toast_custom, null);
        TextView text = layout.findViewById(R.id.text);
        CardView cardView = layout.findViewById(R.id.cardView);
        final ImageView image = layout.findViewById(R.id.image);
        final GifImageView gifImageView = layout.findViewById(R.id.gifImageView);
        switch (call.method) {
            /*
              show toast case
             */
            case "showToast":

                /*
                  Extracted data sent through method channel from flutter code
                 */
                String mMessage = null;
                String length = null;
                String gravity = null;
                if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.KITKAT) {
                    mMessage = Objects.requireNonNull(call.argument("message")).toString();
                    length = Objects.requireNonNull(call.argument("length")).toString();
                    gravity = Objects.requireNonNull(call.argument("gravity")).toString();
                }
                Number bgcolor = call.argument("bgcolor");
                Number textcolor = call.argument("textcolor");
                Number textSize = call.argument("fontSize");
                int radius = call.argument("radius");
                int elevation = call.argument("elevation");
                String icon = call.argument("icon");

                /*
                  Toast Gravity set
                 */
                int mGravity = Gravity.BOTTOM;
                if (gravity != null) {
                    switch (gravity) {
                        case "top":
                            mGravity = Gravity.TOP;
                            break;
                        case "center":
                            mGravity = Gravity.CENTER;
                            break;
                        default:
                            mGravity = Gravity.BOTTOM;
                            break;
                    }
                }

                /*
                  Toast Duration set
                 */
                int mDuration;
                if (length != null && length.equals("long")) {
                    mDuration = Toast.LENGTH_LONG;
                } else {
                    mDuration = Toast.LENGTH_SHORT;
                }

                /*
                  Icon set as per input
                 */
                if (icon == null) {
                    image.setVisibility(View.GONE);
                    gifImageView.setVisibility(View.GONE);
                } else if (icon.equals("close")) {
                    image.setVisibility(View.VISIBLE);
                    gifImageView.setVisibility(View.GONE);
                    image.setImageResource(R.drawable.ic_close);
                } else if (icon.equals("error")) {
                    image.setVisibility(View.VISIBLE);
                    gifImageView.setVisibility(View.GONE);
                    image.setImageResource(R.drawable.ic_error);
                } else if (icon.equals("info")) {
                    image.setVisibility(View.VISIBLE);
                    gifImageView.setVisibility(View.GONE);
                    image.setImageResource(R.drawable.ic_info);
                } else if (icon.equals("success")) {
                    image.setVisibility(View.VISIBLE);
                    gifImageView.setVisibility(View.GONE);
                    image.setImageResource(R.drawable.ic_success);
                } else if (icon.equals("warning")) {
                    image.setVisibility(View.VISIBLE);
                    gifImageView.setVisibility(View.GONE);
                    image.setImageResource(R.drawable.ic_warning);
                } else if (icon.equals("alarm")) {
                    image.setVisibility(View.VISIBLE);
                    gifImageView.setVisibility(View.GONE);
                    image.setImageResource(R.drawable.ic_alarm);
                } else if (icon.equals("location")) {
                    image.setVisibility(View.VISIBLE);
                    gifImageView.setVisibility(View.GONE);
                    image.setImageResource(R.drawable.ic_location);
                } else if (icon.equals("wallet")) {
                    image.setVisibility(View.VISIBLE);
                    gifImageView.setVisibility(View.GONE);
                    image.setImageResource(R.drawable.ic_wallet);
                } else if (icon.equals("dnd")) {
                    image.setVisibility(View.VISIBLE);
                    gifImageView.setVisibility(View.GONE);
                    image.setImageResource(R.drawable.ic_dnd);
                } else if (icon.equals("loading")) {
                    image.setVisibility(View.GONE);
                    gifImageView.setVisibility(View.VISIBLE);
                    gifImageView.setGifImageResource(R.drawable.loading_resize);
                }
                text.setText(mMessage);

                /*
                  Custom style implementation with color
                 */

                if (bgcolor != null) {
                    cardView.setCardBackgroundColor(bgcolor.intValue());
                }
                cardView.setRadius(radius);
                cardView.setCardElevation(elevation);
                if (textSize != null) {
                    text.setTextSize(textSize.floatValue());
                }
                if (textcolor != null) {
                    text.setTextColor(textcolor.intValue());
                }
                int newColor = 0;
                if (textcolor != null) {
                    newColor = textcolor.intValue();
                }
                image.setColorFilter(newColor, PorterDuff.Mode.SRC_ATOP);

                mToast = new Toast(mContext);
                mToast.setDuration(mDuration);
                mToast.setView(layout);

                /*
                  Change gravity location as per the input
                 */
                if (mGravity == Gravity.CENTER) {
                    mToast.setGravity(mGravity, 0, 0);
                } else if (mGravity == Gravity.TOP) {
                    mToast.setGravity(mGravity, 0, 100);
                } else {
                    mToast.setGravity(mGravity, 0, 100);
                }

                mToast.show();
                result.success(true);
                break;

            /*
              cancel case
             */
            case "cancel":
                if (mToast != null) {
                    mToast.cancel();
                }
                result.success(true);
                break;

            /*
              default case
             */
            default:
                result.notImplemented();
                break;
        }
    }
}
